/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "importmediadialog.h"

#include <QTreeWidgetItem>
#include <QTreeWidget>

#include <kdebug.h>
#include <kicon.h>
#include <kio/job.h>
#include <kmessagebox.h>

#include "kbloggermedia.h"
#include "itemsmanager.h"

namespace KBlogger
{

importMediaDialog *importMediaDialog::s_self = 0;

importMediaDialog *importMediaDialog::self(QWidget* parent)
{
    if ( !s_self ) {
        s_self = new importMediaDialog(parent);
    }
    return s_self;
}

importMediaDialog::importMediaDialog(QWidget *parent): QDialog(parent), hasMedia(false)
{
    kDebug();
    setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose);
}

importMediaDialog::~importMediaDialog()
{
    kDebug();
    /*if (cpJob){
      cpJob->kill();
    }*/
    s_self = 0;
    urls.clear();
}

void importMediaDialog::addUrl(const QString& url, const QString& blogname, const QString& postId)
{
    kDebug();
    hasMedia = true;
    Q_ASSERT( !url.isEmpty() );
    Q_ASSERT( !blogname.isEmpty() );

    QTreeWidgetItem *item;
    if ( url.contains("thumb") ) //NOTE fails sometime.
        item = new QTreeWidgetItem( urls.last() ); //Thumbnail
    else
        item = new QTreeWidgetItem(mediaTreeWidget); //No Thumbnail

    item->setText(0, url );
    item->setIcon(0, KIcon("document-import") );
    item->setText(1, blogname );
    item->setText(2, postId );
    mediaTreeWidget->insertTopLevelItem( 0,
                                         item );
    urls += item;
}

void importMediaDialog::exec()
{
    kDebug();
    if ( !hasMedia ) return;
    i = 0;
    mediaData = 0;
    if ( urls.count() )
        QDialog::exec();
}

void importMediaDialog::accept()
{
    kDebug();
    if ( i == urls.count() ) {
        close();
        return;
    }
    //Download the urls in the media cache
    createMediaFromUrl();
}

void importMediaDialog::reject()
{
    kDebug();
    close();
}

void importMediaDialog::createMediaFromUrl()
{
    KUrl url;
    if ( i == urls.count() ) return;
    url = urls.at(i)->text(0);

    if ( mediaData ) delete mediaData;
    mediaData = new QByteArray();

    kDebug() << url;
    Q_ASSERT( url.isValid() );



    if (!fileJob) delete fileJob;
    fileJob = KIO::get( url );

    connect( fileJob, SIGNAL( result( KJob* ) ),
             this, SLOT( handleFileJobResult( KJob* ) ) );
    connect( fileJob, SIGNAL( data( KIO::Job*,  const QByteArray & ) ),
             this, SLOT( handleFileJobData( KIO::Job*,  const QByteArray & ) ) );
    connect( fileJob, SIGNAL( mimetype( KIO::Job*,  const QString & ) ),
             this, SLOT( handleFileJobMimetype( KIO::Job*,  const QString & ) ) );
}

void importMediaDialog::handleFileJobResult( KJob* job)
{
    kDebug() << urls.count();
    Q_ASSERT( mediaData );
    job->disconnect( SIGNAL( data( KIO::Job*,  const QByteArray & ) ) );

    //Get Processed Item;
    QTreeWidgetItem* item;
    if ( i == urls.count() ) return;
    item = urls.at(i);

    if ( job->error() ) {
        //job->showErrorDialog();
        //KMessageBox::sorry ( this, i18n ( "This file is readable, isn't it?"));
        item->setIcon(0, KIcon("dialog-cancel") );
        item->setText(3, i18n("Read Error"));
    } else {
        //TODO Create kbloggerMedia
        kbloggerMedia *media;
        static kbloggerMedia *prev_media;

        KUrl mediaUrl(item->text(0));
        QString filename = mediaUrl.fileName();
        
	if ( (mediaData->count()/1024) > 1024 ){ //Max 1 Mb
	    item->setIcon(0, KIcon("dialog-cancel") );
            item->setText(3, i18n("Size > 1Mb"));
	    mediaData->clear();
	}else{
		media = new kbloggerMedia( *mediaData,
					   filename,
					   mediaMimetype,
					   item->text(1)
					 );
		media->setUrl(mediaUrl);
	
		if ( media->isCached() ) {
		    item->setIcon(0, KIcon("dialog-apply") );
		    item->setText(3, QString("%1 Kb").arg(mediaData->count()/1024));
	
		    //Thumbnail managing
		    if ( prev_media && filename.contains("thumb") ) {
			prev_media->setThumbnail(media);
			media->setFullSizeImage(prev_media);
			prev_media = 0;
		    } else {
			prev_media = media;
		    }
		    //Add Media in the MediaListWidget
		    itemsManager::self()->addMedia(media);
		} else {
		    item->setIcon(0, KIcon("dialog-cancel") );
		    item->setText(3, i18n("CacheFile writing Error"));
		    
		}
	}
    }

    //Try to create a Media from the next Url
    i++;
    createMediaFromUrl();
}

void importMediaDialog::handleFileJobData( KIO::Job* job,  const QByteArray &data)
{
    Q_UNUSED(job);

    kDebug();// << "data:" << data << endl;
    mediaData->append(data);
}

void importMediaDialog::handleFileJobMimetype( KIO::Job* job,  const QString &mimetype )
{
    Q_UNUSED(job);
    kDebug();
    fileJob->disconnect( SIGNAL( mimetype( KIO::Job*,  const QString & ) ) );
    mediaMimetype = mimetype;
}

} //Namespace

#include "importmediadialog.moc"
