/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "medialist.h"

#include <QTreeWidget>
#include <QFile>
#include <QDataStream>
#include <QMenu>

#include <kdebug.h>
#include <kblog/blog.h>
// #include <kblog/blogmedia.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <kblog/blogmedia.h>
#include <kdatetime.h>
#include <kactioncollection.h>
#include <kapplication.h>

#include "kblogger.h"
#include "itemsmanager.h"
#include "kbloggermedia.h"
#include "mainwidget.h"

#define DIR_TO_SAVE_ENTRIES "media/"

#define MAGIC_KEY 0xA1B2C3D5
#define VERSION 002

namespace KBlogger
{
/**
 *  mediaViewList
 */

mediaListWidget::mediaListWidget( const QString& blogname, QWidget* parent): itemList(parent)
{
    kDebug();
    setupUi(this);
    setBlogname(blogname);
}

mediaListWidget::~mediaListWidget()
{
    kDebug();
}

void mediaListWidget::appendMedia( kbloggerMedia* kbMedia )
{
    kDebug();
    Q_ASSERT( kbMedia );

    //Sets status string
    QString status;

    switch ( kbMedia->status() ) {
    case KBlog::BlogMedia::New: {
        status = i18n("Local");
    }
    ; break;
    case KBlog::BlogMedia::Created: {
        status = i18n("Uploaded");
    }
    ; break;
    case KBlog::BlogMedia::Error: {
        status = i18n("Error");
    }
    ; break;

    };

    //TEST
    /*
    QMapIterator<QTreeWidgetItem*, kbloggerMedia*> i(mMediaMap);

    while (i.hasNext()) {

        QTreeWidgetItem *treeItem;
        kbloggerMedia *media;
        i.next();
        treeItem = i.key();
        media = i.value();
        kDebug() << "treeItem" << treeItem
                 << "media" << media;
        if ( media->name() == kbMedia->name() ) { //TODO Use Checksum?
            //if ( kbMedia->status() != KBlog::BlogMedia::Created ) return;
            mMediaMap.remove(treeItem);
            delete treeItem;
            break;
        }
    }*/

    //Creates QTreeWidgetItem for kbMedia
    QTreeWidgetItem *kbMediaItem;
    kbloggerMedia* fullSizeMedia;
    //NOTE kbMedia is a Thumbnail if have a fullSizeImage
    fullSizeMedia = kbMedia->fullSizeImage();

    if ( fullSizeMedia ) {
        kbMediaItem = new QTreeWidgetItem( mMediaMap.key(fullSizeMedia) );
    } else {
        kbMediaItem = new QTreeWidgetItem(listTree);
    }

    kbMediaItem->setIcon (Name, kbMedia->icon() );
    kbMediaItem->setText(Name, kbMedia->name() );
    kbMediaItem->setText(CreationDate,
                         kbMedia->creationDateTime().dateTime().toString("ddd, dd MMM yyyy, HH:mm") );
    kbMediaItem->setText( Size, kbMedia->sizeString() );
    kbMediaItem->setText(Status, status );

    //Appends kbMedia to QMap(key sorted) and Qlist (sorted)
    mMediaMap[kbMediaItem] = kbMedia;
    mSortedMediaList << kbMedia;
}

kbloggerMedia* mediaListWidget::getCurrentMedia()
{
    kDebug();
    if ( !listTree->currentItem() ) return 0;
    return mMediaMap[ listTree->currentItem() ];
}

int mediaListWidget::count()
{
    return mMediaMap.count();
}

/**
 *   mediaList
 */
mediaList::mediaList( const QString& blogname,
                      kblogger* kbloggerParent,
                      QWidget* parent ): mediaListWidget(blogname, parent),
        mKblogger(kbloggerParent)
{
    kDebug();

    itemsManager::self(this)->addInTheWidgetsList(blogname, this);
    mFilename = QString(blogname).prepend(DIR_TO_SAVE_ENTRIES);
    load();

    //Connect
    connect(listTree, SIGNAL( itemSelectionChanged () ),
            SLOT( previewItem() ));
    connect(listTree, SIGNAL(  itemDoubleClicked(QTreeWidgetItem*, int ) ),
            SLOT( openSelectedMedia() ));
    connect(listTree, SIGNAL( customContextMenuRequested(const QPoint&) ),
            SLOT( showPopup(const QPoint&) ));
}

mediaList::~mediaList()
{
    kDebug();
}

void mediaList::appendMedia(kbloggerMedia* kbMedia)
{
    kDebug();
    if ( kbMedia->checksum() == 0 ) return;
    mediaListWidget::appendMedia( kbMedia );

    //
    mainWidget::self()->updateItemNumbers();
}

void mediaList::save()
{
    kDebug();
    Q_ASSERT( !mFilename.isEmpty() );

    //Disable User Input
    KApplication::kApplication()->processEvents(QEventLoop::ExcludeUserInputEvents);

    //Sets file to save
    QString fileToSave;
    fileToSave = KStandardDirs::locateLocal("appdata", mFilename , true);
    kDebug() << "saving file: " << fileToSave;

    //Opens the file in write mode
    QFile file( fileToSave );
    file.open(QIODevice::WriteOnly);
    QDataStream out(&file);   // we will serialize the data into the file

    //Writes header
    out << (quint32)MAGIC_KEY; //"magic number"
    out << (qint32)VERSION; //binary file version

    //Writes File Version
    out.setVersion(QDataStream::Qt_4_0);

    //Writes the number of media to save.
    out << mMediaMap.count();

    //Writes mPostMap data;
    foreach (kbloggerMedia* media, mSortedMediaList) {
        int statusInt = media->status();
        kDebug() << "name=" << media->name();
        kDebug() << "statusInt =" << statusInt;
        kDebug() << "checksum =" << media->checksum();
        if (!media) {
            kError() << "media is NULL" << endl;
            return;
        }

        out << media->name();
        out << media->mimetype();
        out << statusInt;
        out << media->url(); //uploadedMediaUrl
        out << media->checksum();
    }

    //Close file
    file.close();

    //Enable User input
    KApplication::kApplication()->processEvents();
}

void mediaList::load()
{
    kDebug();
    if (mFilename.isEmpty()) {
        kDebug() << "\tmFilename is empty" << endl;
        return;
    }
    mMediaMap.clear(); //FIXME Memory Leak?

    QString fileToOpen = KStandardDirs::locate("appdata", mFilename);
    if (fileToOpen.isEmpty()) {
        kDebug() << "\t the file " <<  mFilename << " doesn't exist" << endl;
        return;
    }
    QFile file( fileToOpen );
    file.open(QIODevice::ReadOnly);
    QDataStream in(&file);

    // Read and check the header
    quint32 magic;
    in >> magic;
    if (magic != MAGIC_KEY) {
        kDebug() << "BAD MAGIC KEY" << endl;
        return;
    }

    // Read the version
    qint32 version;
    in >> version;
    if (version != VERSION) {
        kDebug() << "BAD VERSION" << endl;
        return;
    }

    // Read the number of entries
    qint32 entriesNumber;
    in >> entriesNumber;
    kDebug() << "there are " << entriesNumber << " media saved" << endl;
    //Getting data from file, clear the listTree Widget and insert the readed items in the widget.

    listTree->clear();

    //Thumbnail managing: this var contains fullSizeImage ptr
    kbloggerMedia *prev_media = 0;

    for ( int i = 0; i < entriesNumber; ++i ) {

        QString name, mimetype;
        KUrl uploadedMediaUrl; //Contains the url of a uploaded Media
        int statusInt;
        quint16 checksum;

        //Read data
        in >> name;
        in >> mimetype;
        in >> statusInt;
        in >> uploadedMediaUrl;
        in >> checksum;

        kDebug() << "New Media loaded - checksum" << QString("%1").arg(checksum, 4, 16);

        //create Media
        kbloggerMedia *media;
        media = new kbloggerMedia(name, mimetype, blogname(), checksum);
        media->setStatus( static_cast<KBlog::BlogMedia::Status>(statusInt) );
        media->setUrl(uploadedMediaUrl);

        //Thumbnail managing
        if ( prev_media && name.contains("thumb") ) {
            prev_media->setThumbnail(media);
            media->setFullSizeImage(prev_media);
            prev_media = 0;
        } else {
            prev_media = media;
        }

        appendMedia( media );

    }
    file.close();
}

void mediaList::updateSearchLine()
{
    kDebug();
    Q_ASSERT(mKblogger);
    mKblogger->updateSearchLine(listTree);
}

void mediaList::openSelectedMedia()
{
    kDebug();
    kbloggerMedia* media = getCurrentMedia();
    if (media)
        media->run();
}

QList<kbloggerMedia*> mediaList::getMediaList()
{
    kDebug();
    return mMediaMap.values();
}

void mediaList::removeLocalItem()
{
    kDebug();

    //Gets the ptr of selected media in the widget
    kbloggerMedia* media = getCurrentMedia();
    Q_ASSERT( media );

    //Thumbnail manage
    if ( media->thumbnail() ) {
        KMessageBox::sorry( this, i18n("Remove the thumbnail first"));
        return;
    }
    if ( kbloggerMedia* fsi = media->fullSizeImage() ) {
        fsi->setThumbnail(0);
    }

    //Gets the ptr of the selected QTreeWidgetItem
    QTreeWidgetItem* currentItem = listTree->currentItem();

    //Informs the user that he's deleting a uploaded media
    if ( media->status() != KBlog::BlogMedia::New ) {
        KMessageBox::sorry( this, i18n("You are deleting a cached file.\nKBlogger can't remove the file on the server") );
    }

    //Remove cached file
    if (! media->removeCachedFile() ) {
        KMessageBox::error( this, i18n("Impossible remove the cached file"));
    }

    //Remove from the Map and List
    {
        kbloggerMedia* mediaToRemove;
        mediaToRemove = mMediaMap.value(currentItem);
        Q_ASSERT(mediaToRemove);

        mSortedMediaList.removeAll(mediaToRemove);
        mMediaMap.remove(currentItem);
    }

    //Remove QTreeWidgetItem
    delete currentItem;
    delete media;

    mainWidget::self()->updateItemNumbers();
}

void mediaList::previewItem()
{
    kDebug();
    kbloggerMedia* media = getCurrentMedia();
    if ( !media ) return;
    mainWidget::self()->viewEntry(media);
}

void mediaList::showPopup(const QPoint& pos)
{
    kDebug();

    QTreeWidgetItem * item = 0 ;
    item = listTree->itemAt(pos);
    if (!item) return;

    QMenu menu(i18n("Post menu"), listTree);

    KActionCollection *aktionCollection = mainWidget::self()->actionCollection();
    const QList< QActionGroup * > actionGroups =  aktionCollection->actionGroups();
    QActionGroup* actionGroup;
    for (int i = 0; i < actionGroups.size(); ++i) {
        if ( actionGroups.at(i)->objectName() == "mediaActionGroup" ) {
            actionGroup = actionGroups.at(i);
            break;
        }
    }

    QList<QAction *> actions = actionGroup->actions();
    for (int i = 0; i < actions.size(); ++i) {
        if ( actions.at(i) )
            menu.addAction( actions.at(i) );
    }

    menu.exec(QCursor::pos());
}



}


#include "medialist.moc"
