/***************************************************************************

   Copyright (C) 2007 Antonio Aloisio <gnuton@gnuton.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "postslist.h"

#include <QTableWidget>
#include <QFile>
#include <QVariant>
#include <QTimer>
#include <QMenu>

#include <kdebug.h>
#include <kblog/blog.h>
#include <kblog/blogpost.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <kinputdialog.h>
#include <kactioncollection.h>
#include <kapplication.h>

#include "backend.h"
#include "mainwidget.h"
#include "kblogger.h"
#include "composereditor.h"
#include "kbloggerpost.h"
#include "itemsmanager.h"

#define MAGIC_KEY 0xA1B2C3D4
#define VERSION 001
#define DIR_TO_SAVE_ENTRIES "entries/"

namespace KBlogger
{

/**
 *   postsList
 */
postsList::postsList( const QString& blogName, entriesType type,
                      kblogger* kbloggerParent, QWidget* parent ):
        itemList(parent),
        mBackend(backend::self()),
        mKblogger(kbloggerParent),
        mType(type)
{
    kDebug() << "localDraftPostsList::localDraftPostsList" ;
    mMainWidget = mainWidget::self();
    setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose); //Destroy on close
    setBlogname(blogName);

    setFilename(blogName, type);
    load();
    //true content coloumn
    listTree->setColumnHidden(6, true);

    //Connect
    connect(listTree, SIGNAL( itemSelectionChanged () ),
            SLOT( previewItem() ));
    connect(listTree, SIGNAL(  itemDoubleClicked(QTreeWidgetItem*, int ) ),
            SLOT( modifyPost() ));
    connect(listTree, SIGNAL( customContextMenuRequested(const QPoint&) ),
            SLOT( showPopup(const QPoint&) ));
}

postsList::~postsList()
{
    kDebug() << "postsList::~postsList" ;
}

void postsList::appendPost(KBloggerPost* post)
{
    kDebug() << "postsList::appendPost" ;

    bool jump = false;
    if (!post) return;
    kDebug() << "post=" << post
    << " ID=" << post->postId()
    << " TITLE=" << post->title()
    << " CREATIONDATE=" << post->creationDateTime()
    << " STATUS=" << post->status();
    //TEST
    QMapIterator<QTreeWidgetItem*, KBloggerPost*> i(mPostsMap);

    if ( post->status() == KBlog::BlogPost::Fetched ) {
        if ( itemsManager::self(this)->isStored(post) ) jump = true;
        //NOTE KBlogger policy... All posts in the Sent list
        //     have the status setted to Created.
        post->setStatus(KBlog::BlogPost::Created);
    } else {
        while (i.hasNext()) {
            KBloggerPost *mPost;
            i.next();
            mPost = i.value();
            if ( *mPost == *post ) jump = true;
        }
    }

    if (jump) {
        kDebug() << "Post doesn't added" ;
        return;
    }
    //Change container parameter
    post->setContainer( qobject_cast<QWidget*>(this) );
    //Insert Post in the postsListWidget
    QStringList itemStringList;
    itemStringList
    << post->getBlogName() // coloumn 0
    << post->postId()
    << post->title()
    << post->categories().join(" ")
    << post->creationDateTime().dateTime().toString("ddd, dd MMM yyyy, HH:mm")
    << post->content()
    << QString(); //        << post->publishedString();

    QTreeWidgetItem *item;
    item = new QTreeWidgetItem(itemStringList);
    if ( mType == local )
        item->setIcon( title, post->statusIcon() );
    else
        item->setIcon( id, post->statusIcon() );


    //Insert Post in the mPostsMap
    mPostsMap[item] = post;

    //Insert the item in the listTree
    listTree->insertTopLevelItem(0, item);

    //Update the number of items of the listTree
    mMainWidget->updateItemNumbers();
}

int postsList::count()
{
    kDebug() << "postsList::count() return" << mPostsMap.count() ;

    return mPostsMap.count();
}

void postsList::removePost( KBloggerPost* postToDelete, bool delPost)
{
    kDebug() << "postsList::removePost" ;

    if (!postToDelete) {
        kError() << "postToDelete is NULL";
        return;
    }

    QMap<QTreeWidgetItem*, KBloggerPost*>::iterator it = mPostsMap.begin(), itEnd = mPostsMap.end();
    for ( ; it != itEnd; ++it ) {
        if ( (*it) == postToDelete ) { //if (postInTheWidget->content() == postToDelete->content() ) {
            delete it.key();
            if (delPost )
                delete *it;
            mPostsMap.erase( it );
            break;
        }
    }

    mMainWidget->updateItemNumbers();
}

void postsList::save()
{
    kDebug() << "postsList::save()" ;

    if (mFilename.isEmpty()) {
        kDebug() << "\tmFilename is empty" ;
        return;
    }

    KApplication::kApplication()->processEvents(QEventLoop::ExcludeUserInputEvents);
    QString fileToSave = KStandardDirs::locateLocal("appdata", mFilename , true);
    kDebug() << "\tSave " <<  mPostsMap.count()
    << "entries in " << fileToSave ;
    QFile file( fileToSave );
    file.open(QIODevice::WriteOnly);
    QDataStream out(&file);   // we will serialize the data into the file

    // Write a header
    out << (quint32)MAGIC_KEY; //"magic number"
    out << (qint32)VERSION; //binary file version

    out.setVersion(QDataStream::Qt_4_0);

    //write mPostsMap data;
    out << mPostsMap.count();

    QMapIterator<QTreeWidgetItem*, KBloggerPost*> i(mPostsMap);
    while (i.hasNext()) {
        i.next();
        KBloggerPost *post = i.value();
        if (!post) {
            kError() << "post is NULL" ;
            return;
        }
        out << post->getBlogName();
        out << post->postId();
        out << post->title();
        out << post->categories().join(",");
        out << post->getDateTimeString();
        out << post->content();
        out << post->isPrivate();
    }
    file.close();
    KApplication::kApplication()->processEvents();
}

void postsList::load()
{
    kDebug() << "postsList::open()" ;

    if (mFilename.isEmpty()) {
        kDebug() << "\tmFilename is empty" ;
        return;
    }
    mPostsMap.clear(); //FIXME Memory Leak?

    QString fileToOpen = KStandardDirs::locate("appdata", mFilename);
    if (fileToOpen.isEmpty()) {
        kDebug() << "\t the file " <<  mFilename << " doesn't exist" ;
        return;
    }
    QFile file( fileToOpen );
    file.open(QIODevice::ReadOnly);
    QDataStream in(&file);

    // Read and check the header
    quint32 magic;
    in >> magic;
    if (magic != MAGIC_KEY) {
        kDebug() << "postsList::open() BAD MAGIC KEY" ;
        return;
    }

    // Read the version
    qint32 version;
    in >> version;
    if (version != VERSION) {
        kDebug() << "postsList::open() BAD VERSION" ;
        return;
    }

    // Read the number of entries
    qint32 entriesNumber;
    in >> entriesNumber;

    //Getting data from file, clear the listTree Widget and insert the readed items in the widget.
    QString blogName, postID, title, categories, dateTime, content;
    bool noPublished;
    KDateTime creationDate;
    listTree->clear();

    //while ( !in.atEnd() ){
    for ( int i = 0; i < entriesNumber; ++i ) {
        in >> blogName;
        in >> postID;
        in >> title;
        in >> categories; //CATEGORIES
        in >> dateTime;
        in >> content;
        in >> noPublished;

        KBloggerPost *post = new KBloggerPost(blogName, //FIXME BAD CODE:P
                                              title,
                                              content,
                                              categories.split(","),
                                              noPublished,
                                              creationDate,
                                              postID);

        if ( categories.isEmpty() ) { //FIXME
            QStringList categoriesList;
            categoriesList << QString();
            post->setCategories( categoriesList );
        }

        post->setDateTimeString(dateTime);
        if ( mType == sent )
            post->setStatus(KBlog::BlogPost::Created);
        if ( mType == local )
            post->setStatus(KBlog::BlogPost::New);
        /*        if ( mType == trashed )
                    post->setDeleted(true);
                else
                    post->setDeleted(false);
        */
        appendPost( post );
    }
    file.close();
}

QList<KBloggerPost*> postsList::getEntriesList()
{
    kDebug() << "postsList::getEntriesList()" ;

    return mPostsMap.values();
}

void postsList::previewItem()
{
    kDebug() << "postsList::previewItem" ;
    KBloggerPost* mPost = getCurrentPost();
    if ( !mPost ) {
        kDebug() << "mPost is NULL; It's not a problem if the TreeWidget is empty.";
        return;
    }
    mMainWidget->viewEntry(mPost);
}

void postsList::showPopup(const QPoint& pos)
{
    kDebug();

    QTreeWidgetItem * item = 0 ;
    item = listTree->itemAt(pos);
    if (!item) return;

    QMenu menu(i18n("Post menu"), listTree);

    KActionCollection *aktionCollection = mMainWidget->actionCollection();
    const QList< QActionGroup * > actionGroups =  aktionCollection->actionGroups();
    QActionGroup* actionGroup;
    for (int i = 0; i < actionGroups.size(); ++i) {
        if ( actionGroups.at(i)->objectName() == "postActionGroup" ) {
            actionGroup = actionGroups.at(i);
            break;
        }
    }

    QList<QAction *> actions = actionGroup->actions();
    for (int i = 0; i < actions.size(); ++i) {
        if ( actions.at(i) )
            menu.addAction( actions.at(i) );
    }
    //menu.exec(listTree->mapToGlobal(pos));
    menu.exec(QCursor::pos());
}

void postsList::setFilename( const QString& account, entriesType type )
{
    kDebug() << "postsList::setFilename account='" << account << "'" ;

    QString filename = QString(account).prepend(DIR_TO_SAVE_ENTRIES);
    switch (type) {
    case sent:
        mFilename = filename.append("-sent");
        break;
    case local:
        mFilename = filename.append("-local");
        break;
    case trashed:
        mFilename = filename.append("-trash");
        break;
    }
}

void postsList::trashPost()
{
    kDebug() << "postsList::trashPost()" ;

    //Create the item
    KBloggerPost* post = 0;
    post = getCurrentPost();

    //test
    if (!post) {
        kError() << "post is NULL";
        return;
    }
    /*
        //Add post in the trash widget.
        if (mType != trashed) {
        
    //  post->setContainer( 0 );
        } else if ( post->status() != KBlog::BlogPost::New ) {
            mBackend->removePost(post, mainWidget::self(0, 0) );
        }
    */
    switch ( mType ) {
    case trashed: {
        /*if ( post->status() == KBlog::BlogPost::Error )
        removePost(post, true);
        else*/
        if ( !post->postId().isEmpty() ) {
            //Remove post from the server
            mBackend->removePost( post, mainWidget::self(0, 0) );
        } else {
            //Remove local post from the trash
            removePost(post, true);
        }

    }
    break;

    default:
        //Add post in the trash widget.
        itemsManager::self(this)->trashPost(post);
        //Remove post from the current widget.
        removePost(post, false);
    }
}
void postsList::deletePost()
{
    kDebug();

    KBloggerPost* post = 0;
    post = getCurrentPost();

    //test
    if (!post) {
        kError() << "post is NULL";
        return;
    }

    removePost(post, true);
}

void postsList::modifyPost()
{
    kDebug() ;

    KBloggerPost *post = 0;
    post = mPostsMap[ listTree->currentItem() ];
    if (!post) return;
    mKblogger->modifyPost(post);
}

KBloggerPost* postsList::getCurrentPost()
{
    kDebug() << "postsList::getCurrentPost" ;
    QTreeWidgetItem* currentQTreeItem = listTree->currentItem();
    if (!currentQTreeItem) return 0;
    return mPostsMap[ currentQTreeItem ];
}

void postsList::updateSearchLine()
{
    kDebug() << "postsList::updateKActions" ;
    Q_ASSERT(mKblogger);
    mKblogger->updateSearchLine(listTree);
}
/*
postsList::entriesType postsList::whereIsThisPost()
{
  kDebug();
  return mType;
}
*/
/**
 *  sentPostsList
 */

sentPostsList::sentPostsList( const QString& blogName,
                              kblogger* kbloggerParent, QWidget* parent )
        : postsList( blogName, sent, kbloggerParent, parent)
{
    kDebug() << "sentPostsList::sentPostsList" ;

    itemsManager::self(this)->addInTheWidgetsList(blogName, this);
    listTree->setColumnHidden(blog, true);
    listTree->setColumnHidden(content, true);
}

sentPostsList::~sentPostsList()
{
    kDebug() << "sentPostsList::~sentPostsList" ;
    //mBackend->disconnect();
}

void sentPostsList::updatePostsSentList()
{
    kDebug() << "sentPostsList::updatePostsSentList" ;

//     mBackend->updateBackend(blogname());
    int downloadCount = KInputDialog::getInteger(i18n("Update Sent List"),
                        i18n("How many entries would you like to update?"),
                        10, 1, 50, 1, 0, this);
    if ( downloadCount < 1 ) return;
    mBackend->listPosts(blogname(), downloadCount, mainWidget::self(0, 0) );
}


/**
 *   localDraftPostsList
 */
localDraftPostsList::localDraftPostsList( const QString& blogName,  kblogger* kbloggerParent, QWidget* parent ): postsList(blogName, local, kbloggerParent, parent)
{
    kDebug() << "localDraftPostsList::localDraftPostsList" ;

    itemsManager::self(this)->addInTheWidgetsList(blogName, this);
    listTree->setColumnHidden(blog, true);
    listTree->setColumnHidden(id, true);
    listTree->setColumnHidden(content, true);
}

localDraftPostsList::~localDraftPostsList()
{
    kDebug() << "localDraftPostsList::~localDraftPostsList" ;
}

/**
 *   trashedPostsList
 */
trashedPostsList::trashedPostsList( const QString& blogName,  kblogger* kbloggerParent, QWidget* parent ): postsList(blogName, trashed, kbloggerParent, parent)
{
    kDebug() << "trashedPostsList::trashedPostsList" ;

    itemsManager::self(this)->addInTheWidgetsList(blogName, this);
    listTree->setColumnHidden(blog, true);
    listTree->setColumnHidden(content, true);
}

trashedPostsList::~trashedPostsList()
{
    kDebug() << "trashedPostsList::~trashedPostsList" ;
}

} //namespace
#include "postslist.moc"
